@@ -367,6 +367,9 @@ class S3Connection(AWSAuthConnection):
         if version_id is not None:
             params['VersionId'] = version_id
 
+        if response_headers is not None:
+            params.update(response_headers)
+
         http_request = self.build_base_http_request(method, path, auth_path,
                                                     headers=headers, host=host,
                                                     params=params)
@@ -377,7 +380,7 @@ class S3Connection(AWSAuthConnection):
     def generate_url(self, expires_in, method, bucket='', key='', headers=None,
                      query_auth=True, force_http=False, response_headers=None,
                      expires_in_absolute=False, version_id=None):
-        if self._auth_handler.capability[0] == 'hmac-v4-s3':
+        if self._auth_handler.capability[0] == 'hmac-v4-s3' and query_auth:
             # Handle the special sigv4 case
             return self.generate_url_sigv4(expires_in, method, bucket=bucket,
                 key=key, headers=headers, force_http=force_http,
